using System;
using System.Collections.Generic;
using System.Text;

using System.IO;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace AjaxStyleWebParts
{
  public class AjaxStylePart1 :
  System.Web.UI.WebControls.WebParts.WebPart,
  System.Web.UI.ICallbackEventHandler
  {
    const string GEARS_GIF = "/_layouts/images/GEARS_AN.GIF";
    const string BLANK_GIF = "/_layouts/images/BLANK.GIF";

    //OnLoad places the JavaScript required for the AJAX-style
    //  framework onto the page
    protected override void OnLoad(EventArgs e)
    {
      System.Web.UI.ClientScriptManager csm = Page.ClientScript;
      StringBuilder js = new StringBuilder();

      //General JavaScript functions that can be added by any Web Part
      string SCRIPT_NAME = "GeneralClientCallbackScript";

      if (!csm.IsClientScriptBlockRegistered(SCRIPT_NAME))
      {

        //CallServer will invoke RaiseCallbackEvent server-side out of band
        //  The JavaScript function does not have to be called CallServer
        //.NET will generate a JavaScript WebForm_DoCallback function call
        //  when GetCallbackEventReference is called which wires up all
        //  the callbacks
        //arg will determine which case of the switch
        //  statement in GetCallbackResult will run
        //context needs to be the id of the client-side element
        //  to update with the return string
        js.Append("function CallServer(arg, context)")
          .Append("{ \n")
          .Append("  try \n")
          .Append("  { \n")
          .Append("    var element = document.getElementById(context); \n")
          .Append("    if(element) \n")
          .Append("    { \n")
          .Append("      if(arg != 'Initial') \n")
          .Append("      { \n")
          .Append("        element.innerHTML = 'Loading...'; \n")
          .Append("      } \n")
          .Append(csm.GetCallbackEventReference(
                         this,
                         "arg",
                         "HandleCallbackResult",
                         "context",
                         true)
                 )
          .Append("    } \n")
          .Append("  } \n")
          .Append("  catch(e){window.status = 'ERROR:' + e.Message} \n")
          .Append("} \n");

        //HandleCallbackResult will catch the string returned
        //  The JavaScript function does not have to be called HandleCallbackResult
        //arg will contain the return string from GetCallbackResult
        //context will be the id of the client-side element
        //  to update with the return string
        js.Append("function HandleCallbackResult(arg, context) \n")
          .Append("{ \n")
          .Append("  try \n")
          .Append("  { \n")
          .Append("    var element = \n")
          .Append("      document.getElementById(context); \n")
          .Append("    if(element) \n")
          .Append("    { \n")
          .Append("      element.innerHTML = arg; \n")
          .Append("    } \n")
          .Append("  } \n")
          .Append("  catch(e){window.status = 'ERROR:' + e.Message} \n")
          .Append("} \n");

        csm.RegisterClientScriptBlock(
            this.GetType(),
            SCRIPT_NAME,
            js.ToString(),
            true);
      }

      //Clear the string builder
      js.Remove(0, js.Length);

      //BootWebPart is needed because functions that run when
      //Body Onload is fired cannot have parameters
      //Invoke CallServer with a call to populate this Web Part's innerHTML
      js.Append("function BootWebPart" + this.ID + "() \n")
        .Append("{ \n")
        .Append("  CallServer('Initial', 'ctl00_m_' + '" + this.ID + "'); \n")
        .Append("} \n");

      //Run BootWebPart when the Page has fully loaded
      js.Append("_spBodyOnLoadFunctionNames.push('BootWebPart" + this.ID + "'); \n");

      //Unique JavaScript name for this Web Part script
      SCRIPT_NAME = this.ID + "Script";

      csm.RegisterClientScriptBlock(
          this.GetType(),
          SCRIPT_NAME,
          js.ToString(),
          true);
    }

    //Initial user interface filled with just the spinny
    //  long running process GIF
    //It will be replaced by a call that runs RenderUI
    //  immediately after the page loads
    protected override void CreateChildControls()
    {
      try
      {
        Image img = new Image();
        img.ImageUrl = GEARS_GIF;
        this.Controls.Add(img);
      }
      catch (Exception ex)
      {
        this.Controls.Add(new LiteralControl(ex.Message));
      }
    }


      //Holds the case of the switch statement requested
    private string _eventArgument = "";

    //Initially called by the client-side JavaScript
    //Be sure to remove the sleep function
    public void RaiseCallbackEvent(string eventArgument)
    {
      //Simulate the call taking some time to return
      System.Threading.Thread.Sleep(500);

      //Save aside the function requested
      _eventArgument = eventArgument;
    }

    //Called immediately after RaiseCallbackEvent
    //Generates the string to return to the client
    public string GetCallbackResult()
    {
      StringWriter sWriter = new StringWriter();
      HtmlTextWriter writer = new HtmlTextWriter(sWriter);
      //StringBuilder sb = new StringBuilder();

      Label lbl = null;

      switch (_eventArgument)
      {
        //Called when the page first loads to generate the 
        case "Initial":
          writer = RenderUI(writer);
          break;

        //Called when the Get Server Date button is clicked
        case "GetServerDate":
          lbl = new Label();
          lbl.Text = DateTime.Now.ToLongDateString();
          lbl.ForeColor = System.Drawing.Color.Blue;
          lbl.RenderControl(writer);
          break;

        //Called when the Get Server Time button is clicked
        case "GetServerTime":
          for (int i = 0; i < 10000; i++)
          {
            lbl = new Label();
            lbl.Text = DateTime.Now.ToLongTimeString() + "<br />";
            lbl.ForeColor = System.Drawing.Color.Green;
            lbl.RenderControl(writer);
          }
          break;

        //Never called, this is an exception
        default:
          lbl = new Label();
          lbl.Text = "ERROR: Unknown eventArgument";
          lbl.ForeColor = System.Drawing.Color.Red;
          lbl.RenderControl(writer);
          break;
      }

      return sWriter.ToString();
    }

    //This is the server control code that would usually
    //  get placed into CreateChildControls
    //All the user interface controls should be rendered
    //  the the writer
    private HtmlTextWriter RenderUI(HtmlTextWriter writer)
    {
      Panel div = null;

      Image shim = null;

      Label dateSpan = null;
      Label timeSpan = null;

      Button dateButton = null;
      Button timeButton = null;

      //Create a unique ID
      string dateSpanId = "date" + DateTime.Now.Ticks.ToString();
      string timeSpanId = "time" + DateTime.Now.Ticks.ToString();

      //Div to move entire Web Part user interface down a few pixels
      div = new Panel();

      shim = new Image();
      shim.BorderWidth = Unit.Pixel(0);
      shim.ImageUrl = BLANK_GIF;
      shim.Height = Unit.Pixel(3);
      div.Controls.Add(shim);

      div.RenderControl(writer);

      //Div to output the date button and span on one line
      div = new Panel();
      div.Wrap = false;

      //Button that calls for the server date asynchronously and out of band.
      //The dateSpan id is used to get the element that holds the return string.
      dateButton = new Button();
      dateButton.OnClientClick =
        "CallServer('GetServerDate', '" + dateSpanId
          + "'); return false;";
      dateButton.Text = "Get Server Date";
      dateButton.Width = Unit.Pixel(150);
      div.Controls.Add(dateButton);

      shim = new Image();
      shim.BorderWidth = Unit.Pixel(0);
      shim.ImageUrl = BLANK_GIF;
      shim.Width = Unit.Pixel(10);
      div.Controls.Add(shim);

      //Span that will hold the server date
      dateSpan = new Label();
      dateSpan.Attributes.Add("id", dateSpanId);
      dateSpan.ForeColor = System.Drawing.Color.Blue;
      dateSpan.Text = "Date Span";
      div.Controls.Add(dateSpan);

      div.RenderControl(writer);

      //Div to output the time button and span on one line
      div = new Panel();
      div.Wrap = false;

      //Button that calls for the server date asynchronously and out of band
      //The timeSpan id is used to get the element that holds the return string.
      timeButton = new Button();
      timeButton.OnClientClick =
        "CallServer('GetServerTime', '" + timeSpanId
          + "'); return false;";
      timeButton.Text = "Get Server Time";
      timeButton.Width = Unit.Pixel(150);
      div.Controls.Add(timeButton);

      shim = new Image();
      shim.BorderWidth = Unit.Pixel(0);
      shim.ImageUrl = BLANK_GIF;
      shim.Width = Unit.Pixel(10);
      div.Controls.Add(shim);

      //Span that will hold the server time
      timeSpan = new Label();
      timeSpan.Attributes.Add("id", timeSpanId);
      dateSpan.ForeColor = System.Drawing.Color.Green;
      timeSpan.Text = "Time Span";
      div.Controls.Add(timeSpan);

      div.RenderControl(writer);

      return writer;
    }
  }
}
