:: Usage and Assumptions found at the bottom of this file

:: Establish and report variables
:: Note: the string  :"=  is used at the end of each parameter to strip out
::   any double quotes that may have been passed into the command file
SET AssemblyDir=%1
  ECHO   AssemblyDir="%AssemblyDir:"=%" 
SET AppPool=%2
  ECHO   AppPool="%AppPool:"=%" 
SET WebApplication=%3
  ECHO   WebApplication="%WebApplication:"=%" 
SET WebApplicationDir=%4
  ECHO   WebApplicationDir="%WebApplicationDir:"=%" 
SET ProjectDir=%5
  ECHO   ProjectDir="%ProjectDir:"=%" 
SET TargetDir=%6
  ECHO   TargetDir="%TargetDir:"=%" 
SET TargetName=%7
  ECHO   TargetName="%TargetName:"=%" 
SET TargetVersion=%8
  ECHO   TargetVersion="%TargetVersion:"=%" 
SET TargetCulture=%9
  ECHO   TargetCulture="%TargetCulture:"=%" 

SET TwelveHive=%CommonProgramFiles%\Microsoft Shared\web server extensions\12\
  ECHO   TwelveHive="%TwelveHive%"

ECHO.

:: The GAC|BIN parameter determines the destination of any assembly
:: Since it is first, if it is not present or contains anything other than
:: GAC or BIN (case insensitive), only the commands usage will be shown.
IF /I NOT "%AssemblyDir:"=%" EQU "GAC" (
  IF /I NOT "%AssemblyDir:"=%" EQU "BIN" (
    GOTO :USAGE
  )
)

:: Always change to the root directory of the VS.NET project
:: Everything in the project should be relative to the root directory
IF EXIST "%ProjectDir:"=%" (
  chdir "%ProjectDir:"=%"
  IF NOT ERRORLEVEL == 0 ECHO Could not change to VS.NET project directory && GOTO :ERR
  ECHO.
)

:: Recycle the IIS Application Pool if it is provided
IF NOT "%AppPool:"=%" == "" (
  "%SystemRoot%\system32\cscript.exe" "%SystemRoot%\system32\iisapp.vbs" /a "%AppPool:"=%" /r
  IF NOT ERRORLEVEL == 0 ECHO There was a problem recycling the App Pool && GOTO :ERR
  ECHO.
)

:: Copy project DLLs (if any) from the VS.NET project to:
::   install and force into the GAC or
::   xcopy to the test website's bin
:: It is not an error if there are no DLLs,
::   it just means that there are no DLLs to deploy
IF EXIST "%TargetDir:"=%%TargetName:"=%.dll" (
  ECHO %TargetName:"=%.dll
  IF /I "%AssemblyDir:"=%" EQU "BIN" (
    xcopy "%TargetDir:"=%*.dll" "%WebApplicationDir:"=%%WebApplication:"=%\bin\" /ys
    IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the assembly to the BIN && GOTO :ERR

    IF EXIST "%TargetDir:"=%%TargetName:"=%.pdb" (
      xcopy "%TargetDir:"=%*.pdb" "%WebApplicationDir:"=%%WebApplication:"=%\bin\" /ys
      IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the PDB to the BIN && GOTO :ERR
    )
    ECHO.
  ) ELSE (
    IF /I "%TargetCulture:"=%" EQU "neutral" (
      SET TargetCulture=""

    GOTO :GETKEY
    :GETKEYRETURN
    ECHO --- Public Key Token is: %PublicKeyToken% ---

    "%ProgramFiles%\Microsoft Visual Studio 8\SDK\v2.0\Bin\GacUtil.exe" /nologo /if "%TargetDir:"=%%TargetName:"=%.dll"
    IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the assembly to the GAC && GOTO :ERR

    IF EXIST "%TargetDir:"=%%TargetName:"=%.pdb" (
      xcopy "%TargetDir:"=%%TargetName:"=%.pdb" "%systemroot%\Assembly\GAC_MSIL\%TargetName:"=%\%TargetVersion:"=%_%TargetCulture%_%PublicKeyToken%\"
      IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the PDB to the GAC MSIL && GOTO :ERR
    )
    ECHO.
  )
)

:: Copy all the files from the project's wpcatalog directory to
::   the test website's wpcatalog (including subdirectories)
:: This will copy both dwp and webpart files if they exist
IF EXIST "wpcatalog\*.dwp" (
  xcopy "wpcatalog\*.dwp" "%WebApplicationDir:"=%%WebApplication:"=%\wpcatalog\" /ys
  IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the *.dwp files && GOTO :ERR
  ECHO.
)
IF EXIST "wpcatalog\*.webpart" (
  xcopy "wpcatalog\*.webpart" "%WebApplicationDir:"=%%WebApplication:"=%\wpcatalog\" /ys
  IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the *.webpart files && GOTO :ERR
  ECHO.
)

:: Copy all the files from the project's wpresources directory
::   to the test website's wpresources\[assemblyname] directory (including subdirectories)
IF EXIST "wpresources" (
  IF /I NOT "%AssemblyDir:"=%" EQU "GAC" (
    xcopy "wpresources" "%WebApplicationDir:"=%%WebApplication:"=%\wpresources\%TargetName:"=%\" /ys
    IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the wpresources files && GOTO :ERR
    ECHO.
  ) ELSE (

    :: Copy to the wpresources directory that is a sybling to the 12 Hive.
    :: Requires version and publickeytoken: wpresources\[assemblyname]\[version]_[culture]_[publickeytoken]
    :: [culture] should be emptystring if it is set to neutral
    IF /I "%TargetCulture:"=%" EQU "neutral" (
      SET TargetCulture=""
    )
    xcopy "wpresources" "%TwelveHive:12=wpresources%%TargetName:"=%\%TargetVersion:"=%_%TargetCulture%_%PublicKeyToken%\" /ys
    IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the wpresources files && GOTO :ERR
    ECHO.
  )
)

:: Copy all files from the project's TEMPLATE directory to the 12 Hive (including subdirectories)
IF EXIST "TEMPLATE" (
  xcopy "TEMPLATE" "%TwelveHive%TEMPLATE\" /ys
  IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the 12 Hive TEMPLATE files && GOTO :ERR
  ECHO.
)

:: Copy RESX files
IF EXIST "Resources\*.resx" (
  xcopy "Resources\*.resx" "%TwelveHive%Resources\" /ys
  IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the 12 Hive Resources && GOTO :ERR
  ECHO.
)

:: Copy CAS Config files
IF EXIST "CONFIG\*.config" (
  xcopy "CONFIG\*.config" "%TwelveHive%CONFIG\" /ys
  IF NOT ERRORLEVEL == 0 ECHO There was a problem deploying the 12 Hive CONFIG files && GOTO :ERR
  ECHO.
)

:: Create the Windows SharePoint Services Solution Package (WSP) CAB
IF EXIST "WSP.DDF" (
  MakeCAB /f "WSP.DDF"
  IF NOT ERRORLEVEL == 0 ECHO There was a problem creating the WSP CAB && GOTO :ERR
  ECHO.
)

GOTO :EOF

:GETKEY
ECHO --- Obtaining Public Key Token ---
ECHO e 100 "SET token="             > "%ProjectDir:"=%DEBUG.TMP"
ECHO n PREFIX.TMP                  >> "%ProjectDir:"=%DEBUG.TMP"
FOR %%i in (rcx a w q) do echo %%i >> "%ProjectDir:"=%DEBUG.TMP"
PUSHD ..
:: DEBUG.exe must run from the system32 directory
CHDIR "%SystemRoot%\system32"
"%SystemRoot%\system32\DEBUG.exe" < "%ProjectDir:"=%DEBUG.TMP"
MOVE PREFIX.TMP "%ProjectDir:"=%"
POPD
"%ProgramFiles%\Microsoft Visual Studio 8\SDK\v2.0\Bin\sn" -T "%TargetDir:"=%%TargetName:"=%.dll" | FINDSTR /r /c:"Public key token" > "%ProjectDir:"=%OUTPUT.TMP"
COPY "%ProjectDir:"=%PREFIX.TMP"+"%ProjectDir:"=%OUTPUT.TMP" "%ProjectDir:"=%SETTOKEN.CMD"
CALL "%ProjectDir:"=%SETTOKEN.CMD"
SET PublicKeyToken=%token:~-16%
:: Delete the four temp files
FOR %%f in ("%ProjectDir:"=%DEBUG.TMP" "%ProjectDir:"=%PREFIX.TMP" "%ProjectDir:"=%OUTPUT.TMP" "%ProjectDir:"=%SETTOKEN.CMD") DO DEL %%f > NUL
ECHO --- Obtaining Public Key Token - End ---
GOTO :GETKEYRETURN
:: Return from GETKEY

:ERR
ECHO.
ECHO An error has occured

:USAGE
ECHO PostBuild.cmd Usage:
ECHO.
ECHO Run PostBuild.cmd from the root of a SharePoint VS.NET project.
ECHO Typically, postbuild.cmd will be saved in the root of the VS.NET project
ECHO   and the following command will be entered into the VS.NET project's
ECHO   Post Build Events textbox:
ECHO.
ECHO call "$(ProjectDir)PostBuild.cmd" "BIN" "SharePointAppPool" "Intranet80" "C:\Inetpub\wwwroot\wss\VirtualDirectories\" "$(ProjectDir)" "$(TargetDir)" "$(TargetName)" "1.0.0.0" "neutral"
ECHO.
ECHO It expects a bunch of parameters; the first three are commonly altered,
ECHO   the others do not often change (if using VS.NET macros as anticipated):
ECHO.
ECHO 1. Provide GAC or BIN to determine where the project's DLL (if any) should be deployed.
ECHO      If the project has no DLL, it doesn't matter which value is provided.
ECHO      Switching from GAC to BIN does not remove any previously installed DLLs from the GAC.
ECHO      This parameter will also determine where the wpresources (if any) will be deployed.
ECHO.
ECHO 2. Provide the name of the target WSS Web Application's IIS Application Pool.
ECHO      Put the value in quotes if it contains spaces.
ECHO.
ECHO 3. Provide the name of the folder that contains the target IIS Web Site.
ECHO      SharePoint creates this folder in "C:\Inetpub\wwwroot\wss\VirtualDirectories\"
ECHO      the path to this folder is provided in the fourth parameter.
ECHO      Put the value in quotes if it contains spaces.
ECHO.
ECHO 4. Provide the path to the folder that contains the target IIS Web Site.
ECHO      This is provided seperately from the third parameter because it won't commonly change.
ECHO      Put the value in quotes if it contains spaces.
ECHO.
ECHO Note: VS.NET macros are not available in command files so they must be provided as parameters.
ECHO   While this command can be executed outside of VS.NET, macros are not available.
ECHO.
ECHO 5. Provide the full path to the VS.NET project directory.
ECHO      The VS.NET "$(ProjectDir)" macro is the expected parameter.
ECHO      Put the value in quotes if it contains spaces.
ECHO.
ECHO 6. Provide the full path to the VS.NET target directory.
ECHO      The VS.NET "$(TargetDir)" macro is the expected parameter.
ECHO      Put the value in quotes if it contains spaces.
ECHO.
ECHO 7. Provide the name of the VS.NET assembly to deploy.
ECHO      The VS.NET "$(TargetName)" macro is the expected parameter.
ECHO      Put the value in quotes if it contains spaces.
ECHO.
ECHO 8. Provide the version of the VS.NET assembly to deploy.
ECHO      There isn't a VS.NET macro for this value.
ECHO      This is only used if an assembly is deployed to the GAC.
ECHO      The assembly's public key is determined using an sn command and
ECHO      the culture is assumed to be neutral.
ECHO.
ECHO 9. Provide the culture of the VS.NET assembly to deploy.
ECHO      This is only used if an assembly is deployed to the GAC.
ECHO.
ECHO.
ECHO Assumptions:
ECHO.
ECHO 1. "wpcatalog"
ECHO   *.dwp and *.webpart files will be found in a folder called "wpcatalog"
ECHO   and deployed to "wpcatalog" of the target Web Application
ECHO.
ECHO 2. "wpresources"
ECHO   All WebPart resource files will be found in a folder called "wpresources"
ECHO   and deployed to "wpresources\TargetName" of the target Web Application
ECHO.
ECHO 3. "TEMPLATE"
ECHO   All subdirectories and files found in the VS.NET project's TEMPLATE folder
ECHO   will be deployed to the TEMPLATE folder in the root of the 12 Hive.
ECHO   So, files to be deployed to TEMPLATE must be maintained in the VS.NET project
ECHO   using a mirror of the folder structure found in the root of the 12 Hive.
ECHO   For instance, the ONET.XML file for a CUSTOMSITE site defintion must be
ECHO   located in TEMPLATE\SiteTemplates\CUSTOMSITE\xml\ in the VS.NET project
ECHO   while the webtemp_CUSTOMSITE.xml file must be located in
ECHO   TEMPLATE\1033\XML\. Equally, the feature.xml file for a CUSTOMFEATURE Feature
ECHO   must be located in TEMPLATE\FEATURES\CUSTOMFEATURE\ in the VS.NET project.
ECHO   This is a common best practice even if you do not use this command file.
ECHO.
ECHO 4. "Resources"
ECHO   All common RESX files will be found in the VS.NET project's Resources folder
ECHO   and deployed to the Resources folder in the root of the 12 Hive.
ECHO.
ECHO 5. "CONFIG"
ECHO   All CAS config files will be found in the VS.NET project's CONFIG folder
ECHO   and deployed to the CONFIG folder in the root of the 12 Hive.
ECHO.
ECHO 6. WSP DDF file
ECHO   The Windows SharePoint Services Solution Package (WSP) Directive Definition File (DDF)
ECHO   must be called WSP.DDF (for this script) and must be located in the root
ECHO   of the VS.NET project or it will not be found and creating a solution cab
ECHO   will not be attempted.
ECHO.
ECHO 7. web.config not altered
ECHO Alterations to the target Web Applications web.config are not automated with
ECHO   by this command file.
ECHO.
ECHO If a file/folder is not found in these expected locations,
ECHO it simply is not deployed.
ECHO.