using System;
using System.Collections.Generic;
using System.Text;

using System.Web.UI;
using System.Web.UI.WebControls;

namespace AjaxWebParts
{
  public class AjaxPart1 :
    System.Web.UI.WebControls.WebParts.WebPart
  {
    /// <summary>
    /// Oninit fires before page load. Modifications to the page that are necessary to support Ajax are done here.
    /// </summary>
    protected override void OnInit(EventArgs e)
    {
      base.OnInit(e);

      try
      {
        EnsureScriptManager();
        EnsureUpdatePanelFixups();
      }
      catch (Exception ex)
      {
        this.Controls.Add(new LiteralControl(ex.Message));
      }
    }

    Label label;

    protected override void CreateChildControls()
    {
      base.CreateChildControls();

      try
      {
        Panel div = new Panel();

        Button button = new Button();
        button.Text = "Get Server Time";
        button.Click += new EventHandler(button_Click);
        div.Controls.Add(button);

        div.Controls.Add(new LiteralControl("&nbsp;"));

        label = new Label();
        label.Text = "Time Goes Here";
        div.Controls.Add(label);

//        this.Controls.Add(div);

        //Use an UpdatePanel in code
        UpdatePanel updatePanel = new UpdatePanel();
        updatePanel.ContentTemplateContainer.Controls.Add(div);
        this.Controls.Add(updatePanel);

        //Add a User Control to the User Interface
        UserControl uc =
          Page.LoadControl("_controltemplates/Custom/AjaxControl.ascx")
            as UserControl;
        this.Controls.Add(uc);                                              
      }
      catch (Exception ex)
      {
        this.Controls.Add(new LiteralControl(ex.Message));
      }
    }

    void button_Click(object sender, EventArgs e)
    {
      label.Text = DateTime.Now.ToLongTimeString();
    }


    // Place a ScriptManager on the page if it does not already exist
    private void EnsureScriptManager()
    {
      // Get the existing ScriptManager if it exists on the page
      ScriptManager ajaxScriptManager = ScriptManager.GetCurrent(this.Page);

      if (ajaxScriptManager == null)
      {
        // Create new ScriptManager and EnablePartialRendering
        ajaxScriptManager = new ScriptManager();
        ajaxScriptManager.EnablePartialRendering = true;

        if (this.Page.Form != null)
        {
          // Add the ScriptManager as the first control in the Page.Form
          this.Page.Form.Controls.AddAt(0, ajaxScriptManager);
        }
      }
    }

    // Fixes the problem with postbacks and form actions (DevDiv 55525)
    // Tag:"form" att:"onsubmit" val:"return _spFormOnSubmitWrapper()" 
    // blocks async postbacks after the first one
    // not calling "_spFormOnSubmitWrapper()" breaks all postbacks
    // returning true all the time, somewhat defeats the purpose of the
    // _spFormOnSubmitWrapper() which is to block repetitive postbacks,
    // but it allows MS AJAX Extensions to work properly.
    private void EnsureUpdatePanelFixups()
    {
      if (this.Page.Form != null)
      {
        string formOnSubmitAtt = this.Page.Form.Attributes["onsubmit"];
        if (formOnSubmitAtt == "return _spFormOnSubmitWrapper();")
        {
          this.Page.Form.Attributes["onsubmit"] = "_spFormOnSubmitWrapper();";
        }
      }

      ScriptManager.RegisterStartupScript(this, this.GetType(),
        "UpdatePanelFixup", "_spOriginalFormAction = document.forms[0].action; _spSuppressFormOnSubmitWrapper=true;", true);
    }

  
  }
}
